
// Initialize the QWebChannel
new QWebChannel(qt.webChannelTransport, function (channel) {
    window.pywebview = channel.objects.bridge;
});

function loadCampaignInfo() {
    var select = document.getElementById('campaignSelect');
    var infoDiv = document.getElementById('campaignInfo');
    if (select.value) {
        infoDiv.style.display = 'block';
    } else {
        infoDiv.style.display = 'none';
    }
}

// Function to call the PyQt function for folder selection
function openFolderPicker(inputId) {
    window.pywebview.selectFolder().then(function (folderPath) {
        if (folderPath) {
            document.getElementById(inputId).value = folderPath;
        }
    });
}


// Function to open the save dialog and return the selected file path
function openSaveFileDialog(fileType) {
    return window.pywebview.saveFile(fileType).then(function (filePath) {
        return filePath;
    });
}





function showSection(sectionId) {

    // Remove the "active" class from all links
    document.querySelectorAll('nav a').forEach(link => link.classList.remove('active'));

    // Find the clicked link by matching the sectionId
    const clickedLink = document.querySelector(`nav a[onclick*="${sectionId}"]`);

    // Add the "active" class to the clicked link
    if (clickedLink) {
        clickedLink.classList.add('active');
    }


    // Hide all sections
    const sections = document.querySelectorAll('section');
    sections.forEach(section => {
        section.style.display = 'none';
    });

    // Show the selected section
    const activeSection = document.getElementById(sectionId);
    if (activeSection) {
        activeSection.style.display = 'block';
    }


    if (sectionId === 'accsGroupManager') {
        // Attach event listeners if the section is visible

        loadAccsGroupList("accountGroupSelect");

        document.getElementById('autoloader_container').style.display = 'none';


        setTimeout(function () {
            document.getElementById("accs_warn_info").style.display = "none";
        }, 35000); // 15000 milliseconds = 15 seconds
    }
    else if (sectionId === 'settings') {
        // Attach event listeners if the section is visible
        fetchDeviceId();

        checkLicense(7);

        /*var appSettingPath = document.getElementById("appSetingPath").value;

        if (!appSettingPath) {
            alert("Please select a data store path.");
            return;
        }*/

        // Load existing settings, then update path and save
        loadAppSettings(function (existingSettings) {
            //console.log(existingSettings);
            document.getElementById("appSetingPath").value = existingSettings.mail_data_folder;
            document.getElementById("app_ver_str").innerText = "ColdEmailingJet  " + existingSettings.ver;
            document.getElementById("enable_share_bounce").checked = existingSettings.enable_share_bounce;

            if (existingSettings.seed_list_Folder !== undefined) {
                document.getElementById("seed_list_Path").value = existingSettings.seed_list_Folder;
            }

        });

    }
    else if (sectionId === 'buy') {
        // Attach event listeners if the section is visible
        fetchDeviceId();
    }
    else if (sectionId === 'campaign') {
        loadCampaignList("campaignSelect");
        // Attach event listeners if the section is visible
        //toggleIMAPResponseOptions();

        loadAccsGroupList("accountGroupSelect4camp");

        setTimeout(function () {
            document.getElementById("camp_warn_info").style.display = "none";
        }, 35000); // 15000 milliseconds = 15 seconds

    }
    else if (sectionId === 'reply_manager') {
        loadCampaignList("reply_campaignSelect");
    }
    else if (sectionId === 'spam_checker') {
        //loadCampaignList("reply_campaignSelect");
        loadAppSettings(function (appSettings) {

            document.getElementById("spam_seed_email_addr").textContent = appSettings.sa_seed_mail;
            //const device_key = appSettings.device_key;
            setIframeSrc(appSettings.device_key, appSettings.sa_seed_mail);

        });
    }
    else if (sectionId === 'domain_manager') {
        //loadCampaignList("");
        document.getElementById('track_redirect_rules').innerHTML = '';
        loadDomainCfgList();
    }
    else if (sectionId === 'dkim_manager') {
        //loadCampaignList("");
        //document.getElementById('track_redirect_rules').innerHTML = '';
        loadDkimCfgList();
    }


}


//##################################################################################################
//Util
//##################################################################################################
function showLog(elementId, log_type, log_txt, maxLines = 200) {

    log_txt = log_txt ? log_txt.trim() : log_txt

    // Get the element by its ID
    const element = document.getElementById(elementId);

    // Get current log entries from the element
    const currentLines = element.children;

    // Determine the icon based on log_type
    let iconHTML;
    switch (log_type) {
        case 'info':
            iconHTML = '<i class="fas fa-info-circle" style="color:black; margin-right: 3px;"></i>';
            break;
        case 'success':
            iconHTML = '<i class="fas fa-check-circle" style="color:green; margin-right: 3px;"></i>';
            break;
        case 'error':
            iconHTML = '<i class="fas fa-times-circle" style="color:red; margin-right: 3px;"></i>';
            break;
        default:
            iconHTML = '';
    }

    // Determine row background color for zebra striping (lighter green and white)
    const rowIndex = currentLines.length; // Count total rows currently in element
    const backgroundColor = rowIndex % 2 === 0 ? '#f5fff5' : '#ffffff'; // Light green and white

    // Create the new line with the icon and text
    const newLine = document.createElement("div");
    newLine.style.display = "flex";
    newLine.style.alignItems = "center";
    newLine.style.backgroundColor = backgroundColor;
    newLine.style.padding = "5px 0";
    newLine.innerHTML = `${iconHTML}<span>${log_txt}</span>`;

    // Append the new line to the element
    element.appendChild(newLine);

    // Remove half of the lines if necessary to keep under maxLines
    if (currentLines.length > maxLines) {
        const linesToRemove = Math.floor(maxLines / 2);
        for (let i = 0; i < linesToRemove; i++) {
            element.removeChild(currentLines[0]);
        }
    }

    // Auto-scroll to make the last line visible
    element.scrollTop = element.scrollHeight;
}




window.onload = () => {
    const currentPage = window.location.pathname; // Get the current page path
    //alert(currentPage);
    if (currentPage.includes('buy.html')) {
        showSection('buy');
        return;
    }

    showSection('campaign');

    init_socket_listener();

    set_spam_iframe_observer();
};


let g_socket_io;

function init_socket_listener() {
    g_socket_io = io.connect('http://' + document.domain + ':' + location.port);

    // Set up listeners for socket events
    g_socket_io.on('connect', () => {
        console.log('Connected to the server.');

        // Clear existing listeners to avoid duplicates on reconnect
        g_socket_io.removeAllListeners('mail_sending');
        g_socket_io.removeAllListeners('mail_cleaner');
        g_socket_io.removeAllListeners('sync_track_data');

        // Add listeners for events
        g_socket_io.on('mail_sending', (data) => {
            updateMailSendingUI(data);
        });

        g_socket_io.on('mail_cleaner', (data) => {
            updateMailCleanerUI(data);
        });

        g_socket_io.on('sync_track_data', (data) => {
            updateSyncUI(data);
        });
    });

    g_socket_io.on('disconnect', () => {
        console.log('Disconnected from the server.');
    });
}


function openInBrowser(url) {
    fetch('/open-url', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({ url: url })
    })
        .then(response => response.json())
        .then(data => {
            if (data.error) {
                console.error('Error:', data.error);
            } else {
                console.log(data.message);
            }
        })
        .catch(error => console.error('Error:', error));
}






//##################################################################################################
//Campaign
//##################################################################################################


let g_mail_proc_status = 0;

const ACTION_UNKNOW = 0;
const ACTION_MAIL_SENDING = 1;
const ACTION_IMAP_CHECKING = 2;


let g_action = ACTION_UNKNOW;

let g_cur_camp = "";



// Function to update the UI based on the received data
let repeat_session_count = 0
let auto_session = 0;
function updateMailSendingUI(data) {

    const action = Object.keys(data)[0]; // Get the action (key)
    const value = data[action]; // Get the corresponding value
    //console.log(data);
    if (action === "progress") {
        percent = value;
        if (percent > 100) {
            percent = 100;
        }

        document.getElementById('progressBar').style.width = percent + '%'; // Update the width of the progress bar;

        let info = 'Processing... ' + percent + '% complete';
        //repeat_session_count -= 1;
        if (repeat_session_count > 0) {
            info += ", repeated " + repeat_session_count + " times";
        }

        document.getElementById('mail_log_txt').textContent = info;
    }
    else if (action === "info" || action == "success" || action == "error") {
        //const progressBar = document.getElementById('maillog');
        //progressBar.innerText = value;

        showLog('maillog', action, value);
    }
    else if (action == "status_update") {
        if (value == "Done") {
            g_mail_proc_status = 1;

            document.getElementById('stopProcessBtn').disabled = false;
            document.getElementById('stopProcessBtn').innerText = 'Close';

            document.getElementById('msendingloading').style.display = 'none';

            let info = 'Done'

            if (repeat_session_count > 0) {
                info += ", session repeated " + repeat_session_count + " times";
            }

            document.getElementById('mail_log_txt').textContent = info;

            document.getElementById('progressBar').style.width = '100%';

            loadCampaign(); //refresh info
        }
    }
    else if (action == "summary") {
        document.getElementById('mail_summary').innerText = value;
    }
    else if (action == "repeat_session") {
        repeat_session_count = +value;
    }
    else if (action == "auto_session") {
        auto_session = +value;
        console.log("auto_session " + auto_session)
    }

}





function toggleAdvancedSettings() {
    const advancedSettings = document.getElementById('advancedSettings');
    const toggleIcon = document.getElementById('toggleIcon');

    if (advancedSettings.style.display === 'none') {
        advancedSettings.style.display = 'block';
        toggleIcon.classList.remove('fa-plus');
        toggleIcon.classList.add('fa-minus');
    } else {
        advancedSettings.style.display = 'none';
        toggleIcon.classList.remove('fa-minus');
        toggleIcon.classList.add('fa-plus');
    }
}







function stopProcess() {
    if (g_mail_proc_status == 1) {
        if (auto_session == 1) {
            document.getElementById('stopProcessBtn').innerText = 'Please wait...';
            stopCampaign();
            setTimeout(() => {
                document.getElementById('overlay').style.display = 'none';
            }, 2000); // Exe

            return;
        }

        document.getElementById('overlay').style.display = 'none';
        return;
    }

    document.getElementById('stopProcessBtn').innerText = 'Please wait...';
    document.getElementById('stopProcessBtn').disabled = true;

    //g_cur_camp = campaignName;
    if (g_action == ACTION_MAIL_SENDING) {

        stopCampaign();
    }
    else if (g_action == ACTION_IMAP_CHECKING) {
        stopIMAP();
    }



    console.log("Process stopped.");

}


function showMailSendingUI() {
    g_mail_proc_status = 0;

    document.getElementById('msendingloading').style.display = 'inline';

    document.getElementById('stopProcessBtn').innerText = 'Stop';
    document.getElementById('stopProcessBtn').disabled = false;

    document.getElementById('maillog').innerText = "";

    document.getElementById('overlay').style.display = 'flex';

}




async function startCampaign() {
    const campaignName = document.getElementById('campaignName').value;
    if (!campaignName) {
        alert('Campaign name is required to start the campaign.');
        return;
    }

    document.getElementById('mail_summary').innerText = "";
    document.getElementById('progressBar').style.width = '0%';
    document.getElementById('mail_log_txt').textContent = "";

    showMailSendingUI();

    g_cur_camp = campaignName;
    g_action = ACTION_MAIL_SENDING;

    //alert(g_cur_camp);

    try {
        const response = await fetch('/start_campaign', {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ campaignName: g_cur_camp }),
        });

        const data = await response.json();

        if (data.message) {
            console.log(data.message); // Handle message
        } else if (data.error) {
            //alert()
            alert("Send error:" + data.error); // Handle error
        }
    } catch (error) {
        console.error('Error:', error);
        alert('An error occurred while starting the campaign.');
    }

}




async function stopCampaign() {


    try {
        const response = await fetch('/stop_campaign', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ campaignName: g_cur_camp }),
        });

        if (response.ok) {
            const result = await response.json();
            //alert(result.message);

            return;

            //document.getElementById('overlay').style.display = 'none';
            g_mail_proc_status = 1;
            document.getElementById('stopProcessBtn').innerText = 'Close';
            document.getElementById('stopProcessBtn').disabled = false;

            document.getElementById('msendingloading').style.display = 'none';

        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while stopping the campaign.');
    }
}




async function startIMAP() {
    const campaignName = document.getElementById('campaignName').value;
    if (!campaignName) {
        alert('Campaign name is required to start the campaign.');
        return;
    }

    document.getElementById('mail_summary').innerText = "";
    document.getElementById('progressBar').style.width = '0%';
    document.getElementById('mail_log_txt').textContent = "";

    showMailSendingUI();

    g_cur_camp = campaignName;
    g_action = ACTION_IMAP_CHECKING;

    // Show UI and pass the fetch logic as the callback

    try {
        const response = await fetch('/start_imap', {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ campaignName: g_cur_camp }),
        });

        const data = await response.json();

        if (data.message) {
            console.log(data.message); // Handle message if needed
        } else if (data.error) {
            alert(data.error); // Handle error if present
        }
    } catch (error) {
        console.error('Error:', error);
        alert('An error occurred while starting the campaign.');
    }

}


async function stopIMAP() {


    try {
        const response = await fetch('/stop_imap', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ campaignName: g_cur_camp }),
        });

        if (response.ok) {
            const result = await response.json();
            //alert(result.message);

            return;

            //document.getElementById('overlay').style.display = 'none';
            g_mail_proc_status = 1;
            document.getElementById('stopProcessBtn').innerText = 'Close';
            document.getElementById('stopProcessBtn').disabled = false;

            document.getElementById('msendingloading').style.display = 'none';
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while stopping the campaign.');
    }
}




function addCampaign() {
    const randomNum = Math.floor(1000 + Math.random() * 9000);

    document.getElementById('campaignInfo').style.display = 'block';

    document.getElementById('campaignName').value = 'campaign_' + randomNum;

    document.getElementById('contactsPath').value = '';

    document.getElementById('mailContent').value = '';

    document.getElementById('attachment').value = '';
    document.getElementById('link_path').value = '';
    document.getElementById('skip_domain_path').value = '';
    document.getElementById('include_domain_path').value = '';

    document.getElementById('enableSenderMonitor').checked = false;
    document.getElementById('senderMonitorXAccs').value = 100;
    document.getElementById('senderMonitorYMails').value = 1;

    document.getElementById('totalMailsPerSession').value = 200;
    document.getElementById('threadCount').value = 10;


    document.getElementById('autoRespondWithIMAP').checked = false;

    document.getElementById('enableTrack').checked = false;

    document.getElementById('enableSeed').checked = true;
    document.getElementById('max_seed_send_count').value = 10;

    document.getElementById('enableBcc').checked = false;

    document.getElementById('enableSenderName').checked = false;
    document.getElementById('senderName').value = "";

    document.getElementById('enableRandomContent').checked = false;

    document.getElementById('reply_to_path').value = "";


    document.getElementById('repeatSessionX').value = 0;
    document.getElementById('repeatSessionXMinutes').value = 30;

    document.getElementById('checkBounceAfterSend').checked = false;

    //loadAccsGroupList("accountGroupSelect4camp");
}



let g_camp_accsGroup_name = "";


async function saveCampaign() {

    const campaignName = document.getElementById('campaignName').value.trim();
    const accsGroupName = document.getElementById('accountGroupSelect4camp').value.trim();
    const contactsPath = document.getElementById('contactsPath').value.trim();
    const mailContent = document.getElementById('mailContent').value.trim();
    const attachment = document.getElementById('attachment').value.trim();
    const link_path = document.getElementById('link_path').value.trim();
    const skip_domain_path = document.getElementById('skip_domain_path').value.trim();
    const include_domain_path = document.getElementById('include_domain_path').value.trim();

    const enableSenderMonitor = document.getElementById('enableSenderMonitor').checked;
    const senderMonitorXAccs = +document.getElementById('senderMonitorXAccs').value;
    const senderMonitorYMails = +document.getElementById('senderMonitorYMails').value;


    const totalMailsPerSession = +document.getElementById('totalMailsPerSession').value;
    const threadCount = +document.getElementById('threadCount').value;
    const autoRespondWithIMAP = document.getElementById('autoRespondWithIMAP').checked;
    const imapResponseText = document.getElementById('imapResponseText').value.trim();
    const enableTrack = document.getElementById('enableTrack').checked;
    const enableSeed = document.getElementById('enableSeed').checked;
    const max_seed_send_count = +document.getElementById('max_seed_send_count').value;
    const enableBcc = document.getElementById('enableBcc').checked;
    const enableSenderName = document.getElementById('enableSenderName').checked;
    const senderName = document.getElementById('senderName').value.trim();
    const enableRandomContent = document.getElementById('enableRandomContent').checked;
    const reply_to_path = document.getElementById('reply_to_path').value;
    const repeatSessionX = +document.getElementById('repeatSessionX').value;
    const repeatSessionXMinutes = +document.getElementById('repeatSessionXMinutes').value;

    const checkBounceAfterSend = document.getElementById('checkBounceAfterSend').checked;

    g_camp_accsGroup_name = accsGroupName;

    // Check if required folder paths are empty when not using auto loader
    if (!accsGroupName || !mailContent || !contactsPath) {
        alert('Please choose a account group, or specifiy mail content / mai list folder');
        return;
    }

    if (autoRespondWithIMAP && !imapResponseText) {
        alert("Please enter text for response");
        return;
    }

    const data = {
        campaignName,
        contactsPath,
        mailContent,
        attachment,
        link_path,
        skip_domain_path,
        include_domain_path,

        enableSenderMonitor,
        senderMonitorXAccs,
        senderMonitorYMails,

        totalMailsPerSession,
        threadCount,
        autoRespondWithIMAP,
        imapResponseText,
        enableTrack,
        enableSeed,
        max_seed_send_count,
        enableBcc,
        enableSenderName,
        enableRandomContent,
        senderName,
        reply_to_path,
        accsGroupName,
        repeatSessionX,
        repeatSessionXMinutes,
        checkBounceAfterSend

    };

    try {
        const response = await fetch('/save_campaign', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(data),
        });

        if (response.ok) {
            const result = await response.json();
            alert('Campaign saved successfully!');
            loadCampaignList('campaignSelect'); // Reload campaign list after saving
        } else {
            const errorData = await response.json(); // Get the error message from the response
            alert(`Error: ${errorData.message || errorData.error || 'Failed to save campaign!'}`);
        }

    } catch (error) {
        alert('An error occurred while saving the campaign.');
    }
}



// Function to delete the campaign by calling /del_campaign with campaign name
async function deleteCampaign() {

    const campaignName = document.getElementById('campaignName').value;
    if (!campaignName) {
        alert('Campaign name is required to delete the campaign.');
        return;
    }

    const confirmed = confirm(`Are you sure you want to delete the campaign ${campaignName}`);

    if (!confirmed) {
        return;
    }

    try {
        const response = await fetch('/del_campaign', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ campaignName }),
        });

        if (response.ok) {
            alert('Campaign deleted successfully!');
            loadCampaignList('campaignSelect'); // Reload campaign list after deletion
            document.getElementById('campaignInfo').style.display = 'none'; // Hide the campaign info section
        } else {
            alert('Failed to delete campaign!');
        }
    } catch (error) {
        alert('An error occurred while deleting the campaign.');
    }

}



async function loadCampaign() {
    const selectedCampaign = document.getElementById('campaignSelect').value;

    if (!selectedCampaign) {
        alert('Select a Campaign first');
        document.getElementById('campaignInfo').style.display = 'none'; // Hide campaign info if no campaign is selected
        return;
    }

    g_cur_camp = selectedCampaign;




    //alert(g_cur_camp);

    try {
        const response = await fetch(`/load_campaign/${encodeURIComponent(g_cur_camp)}`);

        if (response.ok) {
            const campaignData = await response.json();

            document.getElementById('campaignName').value = campaignData.campaignName;
            g_camp_accsGroup_name = campaignData.accsGroupName;
            //alert(g_camp_accsGroup_name);

            document.getElementById('contactsPath').value = campaignData.contactsPath;
            document.getElementById('mailContent').value = campaignData.mailContent;

            document.getElementById('attachment').value = campaignData.attachment;
            document.getElementById('link_path').value = campaignData.link_path;
            document.getElementById('skip_domain_path').value = campaignData.skip_domain_path;
            document.getElementById('include_domain_path').value = campaignData.include_domain_path;

            document.getElementById('enableSenderMonitor').checked = campaignData.enableSenderMonitor;
            document.getElementById('senderMonitorXAccs').value = campaignData.senderMonitorXAccs;
            document.getElementById('senderMonitorYMails').value = campaignData.senderMonitorYMails;

            document.getElementById('totalMailsPerSession').value = campaignData.totalMailsPerSession;
            document.getElementById('threadCount').value = campaignData.threadCount;
            document.getElementById('mail_list_count').innerText = campaignData.mail_list_count;

            document.getElementById('autoRespondWithIMAP').checked = campaignData.autoRespondWithIMAP;
            document.getElementById('enableTrack').checked = campaignData.enableTrack;

            document.getElementById('enableSeed').checked = campaignData.enableSeed;
            document.getElementById('max_seed_send_count').value = campaignData.max_seed_send_count;

            document.getElementById('enableBcc').checked = campaignData.enableBcc;

            document.getElementById('enableSenderName').checked = campaignData.enableSenderName;
            document.getElementById('senderName').value = campaignData.senderName;

            document.getElementById('enableRandomContent').checked = campaignData.enableRandomContent;

            document.getElementById('reply_to_path').value = campaignData.reply_to_path;

            document.getElementById('repeatSessionX').value = campaignData.repeatSessionX;
            document.getElementById('repeatSessionXMinutes').value = campaignData.repeatSessionXMinutes;
            document.getElementById('checkBounceAfterSend').checked = campaignData.checkBounceAfterSend;


            toggleEnableSenderMonitor();

            toggleIMAPResponseOptions();

            toggleSeedOptions();

            //toggleBccOptions();

            toggleSenderNameOptions();

            //alert(campaignData.autoRespondWithIMAP);

            document.getElementById('imapResponseText').value = campaignData.imapResponseText;

            // Show the campaign info section
            document.getElementById('campaignInfo').style.display = 'block';


            document.getElementById('accountGroupSelect4camp').value = campaignData.accsGroupName;



        } else {
            alert('Failed to load campaign info!');
        }
    } catch (error) {
        alert(error);
        alert('An error occurred while loading the campaign info.');
    }

}


// Global list to store campaign names
let globalCampaignList = [];

async function loadCampaignList(select_id) {
    try {
        const response = await fetch('/load_campaign_list');

        if (response.ok) {
            const campaignList = await response.json();
            const campaignSelect = document.getElementById(select_id);

            // Reset the dropdown and global list
            campaignSelect.innerHTML = '<option value="">-- Select a Campaign --</option>';
            globalCampaignList = [];

            // Populate the select dropdown and update the global list
            campaignList.forEach(campaign => {
                const option = document.createElement('option');
                option.value = campaign.name;
                option.textContent = campaign.name;
                campaignSelect.appendChild(option);

                // Save the campaign name to the global list
                globalCampaignList.push(campaign.name);
            });

            // Select the option where the text matches g_cur_camp
            if (select_id === "campaignSelect" && g_cur_camp) {
                const matchingOption = Array.from(campaignSelect.options).find(
                    option => option.textContent === g_cur_camp
                );

                if (matchingOption) {
                    campaignSelect.value = matchingOption.value;
                }
            }
        } else {
            alert('Failed to load campaign list!');
        }
    } catch (error) {
        alert('An error occurred while loading the campaign list.');
    }
}


function toggleEnableSenderMonitor() {

    const checkbox = document.getElementById('enableSeed');
    const seedCountContainer = document.getElementById('senderMonitorContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        seedCountContainer.style.display = 'flex'; // Show the text area
    } else {
        seedCountContainer.style.display = 'none'; // Hide the text area
    }
}



function toggleIMAPResponseOptions() {
    const checkbox = document.getElementById('autoRespondWithIMAP');
    const imapResponseContainer = document.getElementById('imapResponseContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        imapResponseContainer.style.display = 'flex'; // Show the text area
    } else {
        imapResponseContainer.style.display = 'none'; // Hide the text area
    }
}

function toggleSeedOptions() {

    const checkbox = document.getElementById('enableSeed');
    const seedCountContainer = document.getElementById('seedCountContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        seedCountContainer.style.display = 'flex'; // Show the text area
    } else {
        seedCountContainer.style.display = 'none'; // Hide the text area
    }
}


/*
function toggleBccOptions() {
    const checkbox = document.getElementById('enableBcc');
    const bccCountContainer = document.getElementById('bccCountContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        bccCountContainer.style.display = 'flex'; // Show the text area
    } else {
        bccCountContainer.style.display = 'none'; // Hide the text area
    }
}*/

function toggleSenderNameOptions() {
    const checkbox = document.getElementById('enableSenderName');
    const bccCountContainer = document.getElementById('senderNameContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        bccCountContainer.style.display = 'flex'; // Show the text area
    } else {
        bccCountContainer.style.display = 'none'; // Hide the text area
    }
}





//##################################################################################################
//Account
//##################################################################################################

let g_accsGroup_name = "";

function getRangeFromInput(id, field_name) {
    const inputField = document.getElementById(id);
    let value = inputField.value.trim(); // Trim to remove any extra spaces

    // Remove any curly braces, square brackets, or parentheses
    value = value.replace(/[{}[\]()]/g, '').trim();

    // Regex to match a single number or a range
    const rangePattern = /^(\d+)(?:,(\d+))?$/;
    const match = value.match(rangePattern);

    if (!match) {
        // Show an alert for invalid input
        alert('Error: Please enter a valid number or range (e.g., 1 or 1,2) for ' + field_name);
        return null; // Return null for invalid input
    }

    // Extract the numbers from the regex match
    const start = parseInt(match[1], 10); // First number
    const end = match[2] ? parseInt(match[2], 10) : start; // Second number or same as first

    if (end < start) {
        // Show an alert for invalid range
        alert('Error: Range is invalid. The end value cannot be less than the start value in ' + field_name);
        return null;
    }

    // If valid, return the range as an array [start, end]
    return [start, end];
}



function setRangeById(id, valueRange) {
    /**
     * Sets a range to an input element by its ID.
     *
     * @param {string} id - The ID of the input element to update.
     * @param {Array<number>} valueRange - A range in the form [start, end] or [start, start].
     * @throws {Error} If the valueRange is invalid or the element is not found.
     */
    const element = document.getElementById(id);
    if (!element || element.tagName !== "INPUT") {
        throw new Error(`Element with ID '${id}' not found or is not an input element.`);
    }

    if (!Array.isArray(valueRange) || (valueRange.length !== 1 && valueRange.length !== 2)) {
        throw new Error("valueRange must be an array with 1 or 2 elements.");
    }

    // Set the input's value based on the range
    if (valueRange.length === 1 || valueRange[0] === valueRange[1]) {
        element.value = valueRange[0].toString(); // Single value
    } else {
        element.value = `${valueRange[0]},${valueRange[1]}`; // Range as "start,end"
    }
}



// Function to get selected values and return as a list
function getSelectedMailCodes() {
    const selectedValues = [];
    // Get all checkboxes with name 'mailcode'
    document.querySelectorAll('input[name="mailcode"]:checked').forEach((checkbox) => {
        selectedValues.push(checkbox.value);
    });
    return selectedValues;
}

// Function to check checkboxes based on a given list of values
function setSelectedMailCodes(values) {
    // Get all checkboxes with name 'mailcode'
    document.querySelectorAll('input[name="mailcode"]').forEach((checkbox) => {
        // Check if the checkbox's value is in the given list
        if (values.includes(checkbox.value)) {
            checkbox.checked = true;
        } else {
            checkbox.checked = false; // Optionally uncheck others
        }
    });
}


function clearProviderUi() {
    const checkboxes = document.querySelectorAll('.mail_provider');

    checkboxes.forEach(checkbox => {
        // Uncheck the checkbox
        checkbox.checked = false;

        // Find the corresponding API token input field
        const apiTokenInput = checkbox.closest('.form-group').querySelector('.api-token');
		const balanceLabel = checkbox.closest('.form-group').querySelector('.loader_balance');

        // Clear the input field
        apiTokenInput.value = '';
		balanceLabel.innerHTML = '';

        
    });
}


function addNewAccsGroup() {
    const randomNum = Math.floor(1000 + Math.random() * 9000);
    document.getElementById('accsGroupName').value = 'AccountGroup_' + randomNum;

    document.getElementById('senderPath').value = '';
    document.getElementById('useAutoLoader').checked = true;
    //document.getElementById('useSmtpRelay').checked = false;
    document.getElementById('max_bcc_count').value = 0;
    //document.getElementById('enableDelayAfterLogin').checked = false;
    document.getElementById('max_delay_login').value = 0;
    document.getElementById('maxSendPerAccount').value = 20;
    //document.getElementById('enableDelaySending').checked = false;
    document.getElementById('max_delay_sending').value = 0;

    document.getElementById('auto_load_apiToken').value = '';
    document.getElementById('auto_load_max_count').value = 10;
    document.getElementById('auto_load_retry_inteval').value = 15;
    document.getElementById('auto_load_max_retry').value = 100;

    document.getElementById('proxyList').value = '';

    document.getElementById('accountGroupInfo').style.display = 'block';

    document.getElementById('accountGroupSelect').value = "";

    //document.getElementById('enableReuseAfterHour').checked = true;
    document.getElementById('reuse_after_hour').value = 0.5;
    document.getElementById('reuse_per_session').value = 1;


    document.getElementById('removeAllAccsBeforeSession').checked = false;
    document.getElementById('autoDelAccount').checked = false;

    document.getElementById('mailAgentSel').value = 'random';

    const checkboxes = document.querySelectorAll('input[name="mailAgent"]');
    checkboxes.forEach(checkbox => {
        checkbox.checked = false;
    });

    document.getElementById('ma_cb_outlook').checked = true;

    document.getElementById('mc_cb_hotmail').checked = true;


    //toggleDelayLoginOptions();
    //toggleDelaySendingOptions();
    //toggleDReuseAfterHourOptions();

    toggleLoaderOptions();

    toggleMailAgentOptions();
	
	clearProviderUi();

}

/*
const providerList = [
    { provider: "hotmail-beemail", api_token: "xxx1", enabled: true },
    { provider: "hotmail-hotmail007", api_token: "xxx2", enabled: false },
    { provider: "hotmail-lution", api_token: "xxx3", enabled: true, balance:111 }
];
*/


function updateApiTokens(providerList) {
    const checkboxes = document.querySelectorAll('.mail_provider');

    checkboxes.forEach(checkbox => {
        const providerData = providerList.find(item => item.provider === checkbox.value);
        const formGroup = checkbox.closest('.form-group');
        const apiTokenInput = formGroup.querySelector('.api-token');
        const balanceLabel = formGroup.querySelector('.loader_balance');

        if (providerData) {
            // Update the API token input
            apiTokenInput.value = providerData.api_token;

            // Update the checkbox state
            checkbox.checked = providerData.enabled;

            // Update the balance label if balance property exists
            if (providerData.balance !== undefined) {
                balanceLabel.innerHTML = `<i class="fas fa-dollar-sign"></i> ${providerData.balance}`;
            } else {
                balanceLabel.innerHTML = ''; // Clear the balance label if no balance is provided
            }
        } else {
            // Clear the API token input and uncheck the checkbox if no provider data is found
            apiTokenInput.value = "";
            checkbox.checked = false;
            balanceLabel.innerHTML = ''; // Clear the balance label
        }
    });
}


function showAccountGroupData(accsGroupData) {
    document.getElementById('accsGroupName').value = accsGroupData.accsGroupName;
    document.getElementById('senderPath').value = accsGroupData.senderPath || '';
    setRangeById("maxSendPerAccount", accsGroupData.maxSendPerAccount);
    //document.getElementById('enableDelayAfterLogin').checked = accsGroupData.enableDelayAfterLogin;
    document.getElementById('max_delay_login').value = accsGroupData.max_delay_login;

    //document.getElementById('enableReuseAfterHour').checked = accsGroupData.enableReuseAfterHour;
    document.getElementById('reuse_after_hour').value = accsGroupData.reuse_after_hour;
    document.getElementById('reuse_per_session').value = accsGroupData.reuse_per_session;

    //document.getElementById('enableDelaySending').checked = accsGroupData.enableDelaySending;
    document.getElementById('max_delay_sending').value = accsGroupData.max_delay_sending;

    setRangeById("max_bcc_count", accsGroupData.max_bcc_count);

    //document.getElementById("auto_load_apiToken").value = accsGroupData.auto_load_apiToken;
    document.getElementById('auto_load_max_count').value = accsGroupData.auto_load_max_count;
    document.getElementById('auto_load_retry_inteval').value = accsGroupData.auto_load_retry_inteval;
    document.getElementById('auto_load_max_retry').value = accsGroupData.auto_load_max_retry;

    document.getElementById('proxyList').value = accsGroupData.proxyList;

    document.getElementById('removeAllAccsBeforeSession').checked = accsGroupData.removeAllAccsBeforeSession;
    document.getElementById('autoDelAccount').checked = accsGroupData.autoDelAccount;

    document.getElementById('mailAgentSel').value = accsGroupData.mailAgentSel;

   
    selectedMailAgents = accsGroupData.selectedMailAgents;
    const checkboxes = document.querySelectorAll('input[name="mailAgent"]');
    checkboxes.forEach(checkbox => {
        // If the checkbox value is in the selectedMailAgents array, check it
        if (selectedMailAgents.includes(checkbox.value)) {
            checkbox.checked = true;
        } else {
            // Otherwise, uncheck it
            checkbox.checked = false;
        }
    });

    selectedMailCodes = accsGroupData.selectedMailCodes;
    setSelectedMailCodes(selectedMailCodes);

    //toggleDelayLoginOptions();
    //toggleDelaySendingOptions();
    //toggleDReuseAfterHourOptions();

    document.getElementById('useAutoLoader').checked = accsGroupData.useAutoLoader;
    //document.getElementById('useSmtpRelay').checked = accsGroupData.useSmtpRelay;

	updateApiTokens(accsGroupData.mail_providers);

    toggleLoaderOptions();

    toggleMailAgentOptions();

    document.getElementById('accountGroupInfo').style.display = 'block';


}


function getCheckedProviders() {
    const checkedProviders = [];
    const checkboxes = document.querySelectorAll('.mail_provider');

    checkboxes.forEach(checkbox => {
        
		
		const apiTokenInput = checkbox.closest('.form-group').querySelector('.api-token');
		
		checkedProviders.push({
			provider: checkbox.value,
			api_token: apiTokenInput.value,
			enabled: checkbox.checked // Include the enabled status
		});
        
    });

    return checkedProviders;
}


function onSaveAccsGroup() {
    const accsGroupName = document.getElementById('accsGroupName').value.trim();
    const senderPath = document.getElementById('senderPath').value.trim();
    const maxSendPerAccount = getRangeFromInput('maxSendPerAccount', "Max send per account");
    //const useSmtpRelay = document.getElementById('useSmtpRelay').checked;
    const useAutoLoader = document.getElementById('useAutoLoader').checked;
    const max_bcc_count = getRangeFromInput('max_bcc_count', "Max BCC count");
    //const enableDelayAfterLogin = document.getElementById('enableDelayAfterLogin').checked;
    const max_delay_login = +document.getElementById('max_delay_login').value;

    //const enableReuseAfterHour = document.getElementById('enableReuseAfterHour').checked;
    const reuse_after_hour = +document.getElementById('reuse_after_hour').value;
    const reuse_per_session = +document.getElementById('reuse_per_session').value;

    //const auto_load_apiToken = document.getElementById('auto_load_apiToken').value.trim();
    const auto_load_max_count = +document.getElementById('auto_load_max_count').value;
    const auto_load_retry_inteval = +document.getElementById('auto_load_retry_inteval').value;
    const auto_load_max_retry = +document.getElementById('auto_load_max_retry').value;
    //const autoload_provider = document.querySelector('input[name="emailAccount"]:checked').value;


	const mail_providers = getCheckedProviders();

    //const enableDelaySending = document.getElementById('enableDelaySending').checked;
    const max_delay_sending = +document.getElementById('max_delay_sending').value;

    const proxyList = document.getElementById('proxyList').value.trim();

    const removeAllAccsBeforeSession = document.getElementById('removeAllAccsBeforeSession').checked;
    const autoDelAccount = document.getElementById('autoDelAccount').checked;

    const mailAgentSel = document.getElementById('mailAgentSel').value;

    const selectedMailAgents = [];
    // Get all checkboxes with name "mailAgent" that are checked
    const checkboxes = document.querySelectorAll('input[name="mailAgent"]:checked');
    // Add the value of each selected checkbox to the list
    checkboxes.forEach(checkbox => {
        selectedMailAgents.push(checkbox.value);
    });


    // Check if required folder paths are empty when not using auto loader
    if (!useAutoLoader && (!senderPath || !accsGroupName)) {
        alert('Please at least provide group name, sender path');
        return;
    }

    if ((mailAgentSel == "manual") && (selectedMailAgents.length === 0)) {
        alert('Please at least select one mail agent');
        return;
    }

    selectedMailCodes = getSelectedMailCodes();


    const data = {
        accsGroupName,
        senderPath,
        maxSendPerAccount,
        useAutoLoader,
        //useSmtpRelay,
        mail_providers,
       //auto_load_apiToken,
        selectedMailCodes,
        auto_load_max_count,
        auto_load_retry_inteval,
        auto_load_max_retry,
        max_bcc_count,
        //enableDelayAfterLogin,
        max_delay_login,
        //enableDelaySending,
        max_delay_sending,
        //enableReuseAfterHour,
        reuse_after_hour,
        reuse_per_session,
        proxyList,
        removeAllAccsBeforeSession,
        autoDelAccount,
        mailAgentSel,
        selectedMailAgents

    };

    saveAccsGroup(data);
}

function onDelAccsGroup() {
    const accsGroupName = document.getElementById('accsGroupName').value.trim();
    removeAccsGroup(accsGroupName);
}



function generateProviderList() {
    const providerList = [];
    const checkboxes = document.querySelectorAll('.mail_provider');

    checkboxes.forEach(checkbox => {
        const formGroup = checkbox.closest('.form-group');
        const apiTokenInput = formGroup.querySelector('.api-token');
        const balanceLabel = formGroup.querySelector('.loader_balance');

        // Extract provider data
        const provider = checkbox.value;
        const api_token = apiTokenInput.value.trim();
        const enabled = checkbox.checked;

        // Extract balance if available
        const balanceText = balanceLabel.textContent.trim();
        const balance = balanceText ? parseFloat(balanceText.replace('$', '')) : undefined;

        // Add provider data to the list
        providerList.push({
            provider,
            api_token,
            enabled,
            balance: isNaN(balance) ? undefined : balance // Only include balance if it's a valid number
        });
    });

    return providerList;
}



async function loadAccsBalance() {
    try {
        const response = await fetch('/load_accs_balance', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                providers: generateProviderList(),
                
            }),
        });

        if (response.ok) {
            const data = await response.json();

            updateApiTokens(data);
        } else {
            console.error("Failed to fetch balance data.");
        }
    } catch (error) {
        console.error("An error occurred while fetching the balance:", error);
    }
}



function toggleLoaderOptions() {
    const autoLoaderCheckbox = document.getElementById('useAutoLoader');
    const autoLoaderContainer = document.getElementById('autoloader_container');
    const senderPathContainer = document.getElementById('senderPathContainer');
    const senderPathInput = document.getElementById('senderPath');
    const browseSenderBtn = document.getElementById('browseSenderBtn');

    if (autoLoaderCheckbox.checked) {
        autoLoaderContainer.style.display = 'block'; // Show auto loader select
        senderPathContainer.style.display = 'none'; // Hide manual input container
        senderPathInput.disabled = true; // Disable input
        browseSenderBtn.disabled = true; // Disable browse button

        document.getElementById('senderPathLabel').style.display = 'none';
		
		const useAutoLoader = document.getElementById('useAutoLoader').checked;
		if (!useAutoLoader){
			return;
		}
		
		loadAccsBalance();


    } else {
        autoLoaderContainer.style.display = 'none'; // Hide auto loader select
        senderPathContainer.style.display = 'flex'; // Show manual input container
        senderPathInput.disabled = false; // Enable input
        browseSenderBtn.disabled = false; // Enable browse button

        document.getElementById('senderPathLabel').style.display = 'flex';
    }
}

/*
function toggleDelayLoginOptions() {
    const checkbox = document.getElementById('enableDelayAfterLogin');
    const delayLogingContainer = document.getElementById('delayLogingContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        delayLogingContainer.style.display = 'flex'; // Show the text area
    } else {
        delayLogingContainer.style.display = 'none'; // Hide the text area
    }
}



function toggleDelaySendingOptions() {
    const checkbox = document.getElementById('enableDelaySending');
    const delaySendingContainer = document.getElementById('delaySendingContainer');

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        delaySendingContainer.style.display = 'flex'; // Show the text area
    } else {
        delaySendingContainer.style.display = 'none'; // Hide the text area
    }
}

function toggleDReuseAfterHourOptions() {
    const checkbox = document.getElementById("enableReuseAfterHour");
    const input_container = document.getElementById("reuseAfterHourContainer");

    // Show or hide the IMAP response options based on the checkbox state
    if (checkbox.checked) {
        input_container.style.display = 'flex'; // Show the text area
    } else {
        input_container.style.display = 'none'; // Hide the text area
    }
}
*/


// Load account groups and populate a select dropdown
async function loadAccsGroupList(select_id) {
    try {
        const response = await fetch('/load_accs_group_list');

        if (response.ok) {
            const loaders = await response.json();
            const selectElement = document.getElementById(select_id);
            selectElement.innerHTML = '<option value="">-- Select an Account Group --</option>'; // Reset options

            // Populate the select dropdown with account groups
            loaders.forEach(loader => {
                const option = document.createElement('option');
                option.value = loader.accsGroupName;
                option.textContent = loader.accsGroupName;
                selectElement.appendChild(option);
            });

            if (select_id == "accountGroupSelect4camp")
                document.getElementById(select_id).value = g_camp_accsGroup_name;
            else {
                document.getElementById(select_id).value = g_accsGroup_name;
                //showAccountGroupData(g_accsGroup_name);
                if (g_accsGroup_name.length > 0) {
                    loadAccsGroup();
                }
            }

        } else {
            alert('Failed to load account groups!');
        }
    } catch (error) {
        alert('An error occurred while loading the account groups.');
    }
}

// Save an account group
async function saveAccsGroup(groupData) {
    try {
        const response = await fetch('/save_accs_group', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(groupData),
        });

        const result = await response.json();
        if (response.ok) {
            alert(result.message);
        } else {
            alert(result.error || 'Failed to save the account group!');
        }
    } catch (error) {
        alert('An error occurred while saving the account group.');
    }
}

// Remove an account group
async function removeAccsGroup(groupName) {
    try {
        const response = await fetch('/remove_accs_group', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ name: groupName }),
        });

        const result = await response.json();
        if (response.ok) {
            alert(result.message);
            loadAccsGroupList("accountGroupSelect");
        } else {
            alert(result.error || 'Failed to remove the account group!');
        }
    } catch (error) {
        alert('An error occurred while removing the account group.');
    }
}

// Load a specific account group
async function loadAccsGroup() {

    const selectElement = document.getElementById('accountGroupSelect');
    g_accsGroup_name = selectElement.value;

    // Ensure a group is selected
    if (!g_accsGroup_name) {
        alert('Please select an account group.');
        return;
    }

    //alert(groupName);


    const response = await fetch('/load_accs_group', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
        },
        body: JSON.stringify({ name: g_accsGroup_name }),
    });

    const result = await response.json();
    if (response.ok) {
        showAccountGroupData(result.data);
        console.log('Loaded group:', result.data); // Handle the loaded group as needed
    } else {
        alert(result.error || `Failed to load account group: ${g_accsGroup_name}`);
    }

}



function toggleMailAgentOptions() {
    const mailAgentSelect = document.getElementById('mailAgentSel');
    const manualMailAgentOptions = document.getElementById('manualMailAgentOptions');

    if (mailAgentSelect.value === 'manual') {
        manualMailAgentOptions.style.display = 'block';
    } else {
        manualMailAgentOptions.style.display = 'none';
    }
}



//##################################################################################################
//Cleaner
//##################################################################################################


var g_clean_proc_status = 0;

function showMailCleanerUI() {
    g_clean_proc_status = 0;
    document.getElementById('stopCleanerBtn').innerText = 'Stop';

    document.getElementById('cleanerlog').innerText = "";

    //initMailCleanerWs();
    document.getElementById('mcleaner_overlay').style.display = 'flex';

    document.getElementById('cleandialog-text').textContent = "";

    document.getElementById('clean_progressBar').style.width = '0%';
}




// Function to update the UI based on the received data
function updateMailCleanerUI(data) {
    const action = Object.keys(data)[0]; // Get the action (key)
    const value = data[action]; // Get the corresponding value

    if (action === "progress") {
        percent = value;
        if (percent > 100) {
            percent = 100;
        }
        document.getElementById('clean_progressBar').style.width = percent + '%';
        document.getElementById('cleandialog-text').textContent = 'Processing... ' + percent + '% complete';
    }
    else if (action === "info" || action == "success" || action == "error") {
        //const progressBar = document.getElementById('maillog');
        //progressBar.innerText = value;

        showLog('cleanerlog', action, value);
    }
    else if (action == "status_update") {
        if (value == "Done") {
            g_clean_proc_status = 1;
            document.getElementById('stopCleanerBtn').innerText = 'Close';
            document.getElementById('clean_progressBar').style.width = '100%'

        }
    }
    else if (action == "summary") {
        document.getElementById('cleaner_summary').innerText = value;
    }


}

async function onStartCleaning() {
    const mailFolder = document.getElementById('senderPathCleaner').value;


    if (!mailFolder) {
        alert('Please provide a valid sender account path.');
        return;
    }

    startCleaning(mailFolder);
}



async function onStartCleaningDB() {
    startCleaning("");
}




async function startCleaning(mailFolder) {

    const checkOnline = document.getElementById('enableBounceOnlineCheck').checked;

    let clean_db = false;
    if (!mailFolder) {
        clean_db = true;
    }


    document.getElementById('cleaner_summary').innerText = "";

    showMailCleanerUI();

    try {
        const response = await fetch('/start_cleaner', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                mail_folder: mailFolder,
                clean_db: clean_db,
                check_online: checkOnline
            }), // Pass all parameters
        });

        if (response.ok) {
            const result = await response.json();
            //alert(result.message); // Show success message
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while starting the cleaning process.');
    }
}




async function stopCleaner() {

    if (g_clean_proc_status == 1) {
        document.getElementById('mcleaner_overlay').style.display = 'none';
        return;
    }

    try {
        const response = await fetch('/stop_cleaner', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({})  // Send an empty object to meet the server's expectation for JSON data
        });

        if (response.ok) {
            const result = await response.json();
            alert(result.message);
            //document.getElementById('mcleaner_overlay').style.display = 'none';
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while stopping the cleaner.');
    }
}





//##################################################################################################
//license
//##################################################################################################



// Function to check the license
async function checkLicense(trialDays) {
    try {
        const response = await fetch('/check_lic', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ trial_days: trialDays }),
        });
        const data = await response.json();
        //console.log(data);
        if (response.ok) {

            document.getElementById('validationResult').innerText = data.message;
        } else {
            alert("Failed to check product data!");
        }
    } catch (error) {
        alert(error);
        alert("Failed to check product data!");
    }
}




// Function to get the unique device ID
async function getDeviceUID() {
    try {
        const response = await fetch('/get_dev_uid', {
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
            },
        });
        const data = await response.json();
        if (response.ok) {
            return { success: true, uniqueId: data.unique_id };
        } else {
            return { success: false, error: data.error };
        }
    } catch (error) {
        return { success: false, error: "Network error" };
    }
}





// Function to fetch the device ID and display it
async function fetchDeviceId() {
    try {
        const result = await getDeviceUID(); // Assuming you already have the getDeviceUID() function
        const deviceIdElem = document.getElementById('deviceId');
        if (result.success) {
            deviceIdElem.textContent = result.uniqueId;
        } else {
            deviceIdElem.textContent = "Error fetching device ID";
        }
    } catch (error) {
        document.getElementById('deviceId').textContent = "Error fetching device ID";
    }
}




//##################################################################################################
//Reply
//##################################################################################################



function onLoadReply() {
    const selectedCampaign = document.getElementById('reply_campaignSelect').value;

    if (!selectedCampaign) {
        alert('Select a Campaign first');
        //document.getElementById('campaignInfo').style.display = 'none'; // Hide campaign info if no campaign is selected
        return;
    }

    loadReplyData(selectedCampaign);


}


async function loadReplyData(camp_name, page = 1) {

    //alert(camp_name);
    try {
        // Create the POST request body
        const requestBody = {
            camp_name: camp_name,
            page: page
        };

        // Send POST request to /reply_manager/ with camp_name and page in the body
        const response = await fetch('/reply_manager/', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(requestBody)
        });

        // Check if response is ok
        if (!response.ok) {
            throw new Error('Network response was not ok');
        }

        // Get the response content and update the HTML
        const htmlContent = await response.text();
        document.getElementById('reply_data_container').innerHTML = htmlContent;
    } catch (error) {
        console.error('There has been a problem with your fetch operation:', error);
        document.getElementById('reply_data_container').innerHTML = 'Failed to load reply data.';
    }
}


function closeReplyEditor() {
    document.getElementById('reply_editor_overlay').style.display = 'none';
}


function onReplyToEmail(data_id) {
    // Fetch reply data by ID
    fetchReplyById(data_id, function (replyData) {
        // Fill in the reply editor fields with the fetched data
        // Set the 'From' and 'To' fields
        document.getElementById('replyFrom').value = replyData.sender_mail;
        document.getElementById('replyTo').value = replyData.from_mail;  // Assuming the 'from_mail' is the recipient

        // Conditionally add "Re:" to the subject if it exists
        if (replyData.subject) {
            document.getElementById('replySubject').value = "Re:" + replyData.subject;
        } else {
            document.getElementById('replySubject').value = "";
        }

        // Conditionally append the original message to the body if it exists
        const originalBodyMessage = "\n\n-------- Original Message --------\n\n";
        if (replyData.body) {
            document.getElementById('replyBody').value = originalBodyMessage + replyData.body;
        } else {
            document.getElementById('replyBody').value = "";
        }

        // Show the reply editor overlay
        document.getElementById('reply_editor_overlay').style.display = 'flex';
        document.getElementById('sendReplyBtn').disabled - false;
        document.getElementById('loadingGif').style.display = 'none';

    });
}



function showReplyEditor(from = '', to = '', subject = '') {
    // Populate the fields with the passed parameters
    document.getElementById('replyFrom').value = from;
    document.getElementById('replyTo').value = to;
    document.getElementById('replySubject').value = subject;

    // Clear the body in case it was used before
    document.getElementById('replyBody').value = '';

    // Display the overlay
    document.getElementById('reply_editor_overlay').style.display = 'flex'; // 'flex' to align overlay elements in the center
}



// Function to send email data to the Flask API
function sendEmail(mail_prop) {
    return fetch('/send_email/', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(mail_prop)
    })
        .then(response => {
            if (!response.ok) {
                throw new Error('Failed to send email: ' + response.statusText);
            }
            return response.json();
        });
}

// Function to handle reply sending
function sendReply() {
    const sendButton = document.getElementById('sendReplyBtn');
    const loadingGif = document.getElementById('loadingGif');

    const from_mail = document.getElementById('replyFrom').value;
    const to_mail = document.getElementById('replyTo').value;
    const subject = document.getElementById('replySubject').value;
    const body = document.getElementById('replyBody').value;
    const selectedCampaign = document.getElementById('reply_campaignSelect').value;
    // Validate input fields
    if (!from_mail || !to_mail || !subject || !body) {
        alert("All fields are required!");
        return;
    }

    // Prepare the email data
    const mail_prop = {
        from_mail: from_mail,
        to: to_mail,
        subject: subject,
        mail_body: body,
        camp_name: selectedCampaign
    };

    // Disable the send button and show loading GIF
    sendButton.disabled = true;
    loadingGif.style.display = 'inline';

    // Call the API to send the email
    sendEmail(mail_prop)
        .then(data => {
            console.log('Email sent successfully!', data);
            alert(data.message); // Display success message

            // Close editor after sending
            closeReplyEditor();
        })
        .catch(error => {
            console.error('Error:', error);
            alert("Failed to send email: " + error.message);
        })
        .finally(() => {
            // Hide loading GIF and enable the send button again
            loadingGif.style.display = 'none';
            sendButton.disabled = false;
        });
}





function fetchReplyById(replyId, callback) {
    // Prepare the data to be sent in the POST request
    const requestData = {
        id: replyId
    };

    // Make the POST request to the Flask API
    fetch('/fetch_reply_by_id/', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
        },
        body: JSON.stringify(requestData),
    })
        .then(response => {
            if (!response.ok) {
                alert('Failed to fetch reply data');
            }
            return response.json();
        })
        .then(data => {
            console.log('Reply Data:', data);
            // Call the provided callback function and pass the reply data to it
            callback(data);
        })
        .catch(error => {
            console.error('Error:', error);
        });
}


function onExportReply() {
    const selectedCampaign = document.getElementById('reply_campaignSelect').value;

    if (!selectedCampaign) {
        alert('Select a Campaign first');
        //document.getElementById('campaignInfo').style.display = 'none'; // Hide campaign info if no campaign is selected
        return;
    }

    openSaveFileDialog('csv').then(function (filePath) {
        if (filePath) {
            console.log("Selected file path:", filePath);
            exportReplyData(selectedCampaign, filePath);
        }
    });


}



async function exportReplyData(camp_name, filePath) {
    try {
        const response = await fetch('/export_reply_data/', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ camp_name, file_path: filePath })
        });

        const result = await response.json();

        if (response.ok) {
            console.log(result.message);
            alert(result.message); // Show success message
        } else {
            console.error(result.error);
            alert('Error: ' + result.error); // Show error message
        }
    } catch (error) {
        console.error('Error exporting reply data:', error);
        alert('An error occurred while exporting the data.');
    }
}




//##################################################################################################
//Settings
//##################################################################################################


function onSaveAppSettings() {
    var appSettingPath = document.getElementById("appSetingPath").value;

    //alert(appSettingPath);
    if (!appSettingPath) {
        alert("Please select a data store path.");
        return;
    }

    var seed_path = document.getElementById("seed_list_Path").value;
    var enable_share_bounce = document.getElementById("enable_share_bounce").checked;



    // Load existing settings, then update path and save
    loadAppSettings(function (existingSettings) {
        existingSettings.mail_data_folder = appSettingPath;

        existingSettings.seed_list_Folder = seed_path;
        existingSettings.enable_share_bounce = enable_share_bounce;

        saveAppSettings(existingSettings); // Save updated settings
    });
}


function loadAppSettings(callback) {
    fetch('/load_app_settings')
        .then(response => response.json())
        .then(data => {
            if (callback) {
                callback(data); // Pass the settings to the callback function
            }
        })
        .catch(error => {
            console.error("Error loading settings:", error);
            alert("An error occurred while loading app settings.");
        });
}

function saveAppSettings(appData) {
    fetch('/save_app_settings', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(appData) // Send the entire app data
    })
        .then(response => response.json())
        .then(data => {
            if (data.error) {
                alert("Error saving settings: " + data.error);
            } else {
                alert("Settings saved successfully!");
            }
        })
        .catch(error => {
            console.error("Error:", error);
            alert("An error occurred while saving settings.");
        });
}





//##################################################################################################
//Stat
//##################################################################################################
async function loadStatData(callback) {
    const selectedCampaign = document.getElementById('campaignSelect').value;

    try {
        const response = await fetch(`/load_stat_data/?camp_name=${encodeURIComponent(selectedCampaign)}`, {
            method: 'GET',
            headers: { 'Content-Type': 'application/json' }
        });

        if (!response.ok) {
            console.error("Error loading stat data");
            return;
        }

        const data = await response.json();
        //console.log("Loaded Stat Data:", data);

        // Invoke the callback with the loaded data
        if (typeof callback === 'function') {
            callback(data);
        }
    } catch (error) {
        console.error("Error loading stat data:", error);
    }
}

let statChart = null; // Store the chart instance globally

function renderStatChart(data) {
    // Destroy the existing chart instance if it exists
    if (statChart) {
        statChart.destroy();
    }

    // Prepare labels and datasets
    const labels = [...new Set(data.map(record => record.event_time))]; // Unique timestamps

    // Format the labels to MMDD-hh:mm
    const formattedLabels = labels.map(time => {
        const date = new Date(time); // Convert the time string to a Date object
        const month = String(date.getMonth() + 1).padStart(2, '0'); // Months are 0-based
        const day = String(date.getDate()).padStart(2, '0');
        const hours = String(date.getHours()).padStart(2, '0');
        const minutes = String(date.getMinutes()).padStart(2, '0');
        return `${month}${day}-${hours}:${minutes}`; // Format to MMDD-hh:mm
    });

    const datasets = [];

    // Grouping data by event_type
    const eventTypes = [...new Set(data.map(record => record.event_type))];

    eventTypes.forEach(eventType => {
        const eventData = data.filter(record => record.event_type === eventType);
        const values = formattedLabels.map(time => {
            const record = eventData.find(r => {
                // Convert record event_time to the same format for comparison
                const recordDate = new Date(r.event_time);
                const recordMonth = String(recordDate.getMonth() + 1).padStart(2, '0');
                const recordDay = String(recordDate.getDate()).padStart(2, '0');
                const recordHours = String(recordDate.getHours()).padStart(2, '0');
                const recordMinutes = String(recordDate.getMinutes()).padStart(2, '0');
                return `${recordMonth}${recordDay}-${recordHours}:${recordMinutes}` === time;
            });
            return record ? record.event_value : 0; // Return 0 if no record for this time
        });

        datasets.push({
            label: eventType,
            data: values,
            backgroundColor: getColor(eventType), // Use color function
            borderColor: getColor(eventType),
            borderWidth: 2,
            fill: true // Fill the area under the line
        });
    });

    // Create the chart
    const ctx = document.getElementById('statChart').getContext('2d');
    statChart = new Chart(ctx, { // Assign to global variable
        type: 'line', // Line chart
        data: {
            labels: formattedLabels, // Use the formatted labels
            datasets: datasets
        },
        options: {
            responsive: true,
            scales: {
                x: {
                    title: {
                        display: true,
                        text: 'Event Time'
                    }
                },
                y: {
                    beginAtZero: true,
                    title: {
                        display: true,
                        text: 'Event Value (Count)'
                    }
                }
            },
            plugins: {
                legend: {
                    display: true,
                    position: 'top'
                }
            }
        }
    });
}



// Helper function to get color based on event type
function getColor(eventType) {
    const colors = {
        'Bounced': 'rgba(255, 99, 132, 0.2)', // Red
        'Replied': 'rgba(54, 162, 235, 0.2)', // Blue
        'Sent': 'rgba(75, 192, 192, 0.2)', // Teal
        'Blocked': 'rgba(255, 206, 86, 0.2)', // Yellow
        'Open': 'rgba(153, 102, 255, 0.2)', // Purple
        'Click': 'rgba(255, 159, 64, 0.2)', // Orange
        'Unsub': 'rgba(255, 205, 86, 0.2)' // Light Yellow
    };

    // Compare eventType in lowercase with the keys
    const lowerCasedEventType = eventType.toLowerCase();

    for (const key of Object.keys(colors)) {
        if (key.toLowerCase() === lowerCasedEventType) {
            return colors[key]; // Return the corresponding color
        }
    }

    return 'rgba(0, 0, 0, 0.2)'; // Default color
}




function onShowCampChart() {

    document.getElementById('campstat_overlay').style.display = 'flex';

    loadStatData(renderStatChart);
}


function onStopCampStatUI() {
    // Hide the overlay
    document.getElementById('campstat_overlay').style.display = 'none';

}






//##################################################################################################
//Spam checker
//##################################################################################################

function set_spam_iframe_observer() {

    const iframe = document.getElementById('spamIframe');

    const observer = new IntersectionObserver(entries => {
        entries.forEach(entry => {
            if (entry.isIntersecting) {
                // Load iframe content only when it's visible
                iframe.src = iframe.getAttribute('data-src');
                observer.disconnect(); // Stop observing once iframe is loaded
            }
        });
    }, { threshold: 0.5 });

    observer.observe(iframe);
}


function onRefreshSpamChecker() {
    const iframe = document.getElementById('spamIframe');
    const iframeSrc = iframe.getAttribute('data-src'); // Get the original source URL

    // Temporarily change the iframe's src to refresh it
    iframe.src = '';  // Clear the current src
    iframe.src = iframeSrc;  // Reset the src to the original URL
}


function setIframeSrc(deviceKey, email) {
    const iframe = document.getElementById("spamIframe");
    if (!iframe) {
        console.error("Iframe not found!");
        return;
    }

    const baseUrl = "https://topoffer.monster/spam_checker.php";
    const url = `${baseUrl}?action=getmaillist&device_key=${encodeURIComponent(deviceKey)}&email=${encodeURIComponent(email)}`;
    iframe.setAttribute("data-src", url);
    iframe.setAttribute("src", url); // Load immediately
}




//##################################################################################################
//Tracking domain 
//##################################################################################################

let ruleCounter = 0;


function addRedirectRule(url = '', selectedCamp = null) {
    const campList = globalCampaignList; // Assuming this is defined globally
    const trackRedirectRules = document.getElementById("track_redirect_rules");

    // Check if the number of existing rules equals the campaign list length
    if (trackRedirectRules.children.length >= campList.length) {
        alert("You cannot add more rules. Only one rule for each camp is allowed");
        return;
    }

    const ruleDiv = document.createElement("div");
    ruleDiv.id = `rule_${ruleCounter}`;
    ruleDiv.className = "rule-item";

    // Create the options for the select element, marking the selected one if provided
    const options = campList.map((camp) => {
        const isSelected = selectedCamp === camp ? 'selected' : '';
        return `<option value="${camp}" ${isSelected}>${camp}</option>`;
    }).join('');

    //alert(url);
    // Set the inner HTML with the provided URL and generated options
    ruleDiv.innerHTML = `
        <input type="text" id="track_unsub_txt_${ruleCounter}" class="rule-input" 
            placeholder="Enter URL here" value="${url}">

        <label for="accountGroupSelect_${ruleCounter}" class="rule-label">
            for camp
        </label>
        <select id="accountGroupSelect_${ruleCounter}" class="rule-select">
            ${options}
        </select>

        <button class="delete-button" onclick="removeRule('${ruleDiv.id}')">
            <i class="fas fa-trash-alt"></i> Delete
        </button>
    `;

    trackRedirectRules.appendChild(ruleDiv);
    ruleCounter++;
}




function validateAndExtractRules() {
    const rules = document.querySelectorAll('.rule-item');
    const result = [];

    for (const rule of rules) {
        const input = rule.querySelector('.rule-input');
        const select = rule.querySelector('.rule-select');

        const url = input.value.trim();
        const campName = select.value;

        // Validate URL
        try {
            new URL(url); // Throws an error if the URL is invalid
        } catch {
            alert(`Invalid URL in rule`);
            return false;
        }

        // Validate campaign selection
        if (!campName) {
            alert(`No campaign selected for rule: ${select.id}`);
            return false;
        }

        result.push({ url, camp_name: campName });
    }

    return result;
}




function removeRule(ruleId) {
    const ruleElement = document.getElementById(ruleId);
    if (ruleElement) {
        ruleElement.remove();
    }
}


let g_selected_domain = "";

async function loadDomainCfgList() {
    try {
        const response = await fetch('/load_domain_cfg_list');
        if (response.ok) {
            const result = await response.json();

            // Populate the select element
            const domainSelect = document.getElementById('domainSelect');
            domainSelect.innerHTML = '<option value="">-- Select a Domain --</option>'; // Reset options

            result.forEach(domain => {

                const option = document.createElement('option');
                option.value = domain.domain_name; // Set the value of the option
                option.textContent = domain.domain_name; // Set the display text of the option
                domainSelect.appendChild(option);
            });

            document.getElementById('domainSelect').value = g_selected_domain;

            console.log(result); // Optionally handle the result for debugging or additional actions
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while loading the domain config list.');
    }
}



function onLoadDomainLCfg() {
    g_selected_domain = document.getElementById('domainSelect').value;
    if (!g_selected_domain) {
        document.getElementById('domainCfgInfo').style.display = 'none';
        return;
    }
    loadDomainCfg(g_selected_domain);
}

async function loadDomainCfg(name) {
    try {
        const response = await fetch(`/load_domain_cfg/${encodeURIComponent(name)}`);
        if (response.ok) {
            const result = await response.json();
            showDomainCfg(result);
            console.log(result); // Handle the result as needed
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.message);
        }
    } catch (error) {
        alert(`An error occurred while loading the domain config for ${name}.`);
    }
}

async function saveDomainCfg(data) {
    try {
        document.getElementById('track_sync_log').innerText = "Please wait...";
        const response = await fetch('/save_domain_cfg', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(data),
        });

        if (response.ok) {
            const result = await response.json();
            alert(result.message);
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.message);
        }
        document.getElementById('track_sync_log').innerText = "";
    } catch (error) {
        alert('An error occurred while saving the domain config.');
    }
}

async function removeDomainCfg(domain_name) {
    if (!domain_name) {
        alert('Campaign name is required.');
        return;
    }

    try {
        const response = await fetch('/remove_domain_cfg', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ domain_name }),
        });

        if (response.ok) {
            const result = await response.json();
            loadDomainCfgList();
            document.getElementById('domainCfgInfo').style.display = 'none';
            g_selected_domain = "";
            document.getElementById('domainCfgInfo').style.display = 'none';
            alert(result.message);
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert(`An error occurred while removing the domain config for ${domain_name}.`);
    }
}





function addNewDomainCfg() {

    document.getElementById('domainCfgInfo').style.display = 'block';

    document.getElementById('domain_name').value = '';
    document.getElementById('cf_api_token').value = '';
    document.getElementById('cf_zone_id').value = "";
    document.getElementById('cf_account_id').value = "";
    document.getElementById('track_unsub_txt').value = "Your will not receive our message again";
    document.getElementById('enable_domain_tracking').checked = true;

    addRedirectRule();

}


function showDomainCfg(domainCfgData) {

    document.getElementById('domainCfgInfo').style.display = 'block';

    document.getElementById('domain_name').value = domainCfgData.domain_name;
    document.getElementById('cf_api_token').value = domainCfgData.cf_api_token;
    document.getElementById('cf_zone_id').value = domainCfgData.cf_zone_id;
    document.getElementById('cf_account_id').value = domainCfgData.cf_account_id;
    document.getElementById('track_unsub_txt').value = domainCfgData.track_unsub_txt;
    document.getElementById('enable_domain_tracking').checked = domainCfgData.enable_domain_tracking;

    for (let i = 0; i < domainCfgData.redirect_rules.length; i++) {
        const redirectRule = domainCfgData.redirect_rules[i];
        //console.log(redirectRule);
        addRedirectRule(redirectRule.url, redirectRule.camp_name);
    }

}


function validatedomain_name(domain) {
    // Check if the input is empty or only whitespace
    if (!domain || domain.trim() === "") {
        return false; // Invalid: Domain is empty
    }

    const domainRegex = /^(?!:\/\/)([a-zA-Z0-9-_]+\.)+[a-zA-Z]{2,}$/;

    // Check against the domain regex
    if (domainRegex.test(domain.trim())) {
        return true; // Valid domain
    } else {
        return false; // Invalid domain
    }
}




function onSaveDomainCfg() {

    const domain_name = document.getElementById('domain_name').value;
    const cf_api_token = document.getElementById('cf_api_token').value;
    const cf_zone_id = document.getElementById('cf_zone_id').value;
    const cf_account_id = document.getElementById('cf_account_id').value;
    const track_unsub_txt = document.getElementById('track_unsub_txt').value;
    const enable_domain_tracking = document.getElementById('enable_domain_tracking').checked;

    if (!validatedomain_name(domain_name)) {
        alert("Invalid domain name");
        return;
    }


    if (!cf_api_token || !cf_zone_id) {
        alert("Please enter cloudflare api token or zone id");
        return;
    }

    const redirect_rules = validateAndExtractRules();
    if (!redirect_rules) {
        console.log('Validation failed.');
        return;
    }


    const data = {
        domain_name,
        cf_api_token,
        cf_zone_id,
        cf_account_id,
        track_unsub_txt,
        enable_domain_tracking,
        redirect_rules

    };

    saveDomainCfg(data);

}


function onDelDomainCfg() {
    g_selected_domain = document.getElementById('domainSelect').value;
    if (!g_selected_domain) {
        alert("Please select a domain cfg firt");
        return;
    }

    removeDomainCfg(g_selected_domain);
}




function onExportTrackData() {

    openSaveFileDialog('csv').then(function (filePath) {
        if (filePath) {
            console.log("Selected file path:", filePath);
            exportTrackData(filePath);
        }
    });


}



async function exportTrackData(filePath) {
    try {
        const response = await fetch('/export_track', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ file_path: filePath })  // pass file path to Flask API
        });

        const result = await response.json();
        if (response.ok) {
            console.log(result.message);
            alert(result.message);
        } else {
            console.error(result.error);
            alert('Error: ' + result.error);
        }
    } catch (error) {
        console.error('Error exporting track data:', error);
    }
}



// Function to update the UI based on the received data
function updateSyncUI(data) {

    const action = Object.keys(data)[0]; // Get the action (key)
    const value = data[action]; // Get the corresponding value
    console.log(data);
    if (action === "progress") {
        percent = value;
        if (percent > 100) {
            percent = 100;
        }

    }
    else if (action === "info" || action == "success" || action == "error" || action == "summary") {
        //const progressBar = document.getElementById('maillog');
        //progressBar.innerText = value;

        document.getElementById('track_sync_log').innerText = value;
    }
    else if (action == "status_update") {
        if (value == "Done") {
            g_mail_proc_status = 1;
            document.getElementById('syncTrackBtn').disabled = false;

            //document.getElementById('mail_log_txt').textContent = 'Done';
        }
    }



}



function onSyncTrackData() {

    //alert("OK");

    document.getElementById('syncTrackBtn').disabled = true;
    syncTrackData();
}



function syncTrackData() {
    const url = '/sync_track_info';  // Flask API endpoint

    fetch(url, {
        method: 'POST',  // Send a POST request
        headers: {
            'Content-Type': 'application/json',  // Set content type to JSON
        }
    })
        .then(response => response.json())  // Parse JSON response
        .then(result => {
            if (result.message) {
                console.log(result.message);  // Log success message
                //alert('Sync started successfully!');
            } else if (result.error) {
                console.error(result.error);  // Log any error
                alert('Failed to start sync:' + result.error);
            }
        })
        .catch(error => {
            console.error('Error:', error);  // Handle network or other errors
            alert('An error occurred!');
        });
}





//##################################################################################################
//DKIM 
//##################################################################################################


let g_selected_dkim_domain = "";

async function loadDkimCfgList() {
    try {
        const response = await fetch('/load_dkim_cfg_list');
        if (response.ok) {
            const result = await response.json();

            // Populate the select element
            const domainSelect = document.getElementById('dkimDomainSelect');
            domainSelect.innerHTML = '<option value="">-- Select a Domain --</option>'; // Reset options

            result.forEach(domain => {

                const option = document.createElement('option');
                option.value = domain.dkim_domain_name; // Set the value of the option
                option.textContent = domain.dkim_domain_name; // Set the display text of the option
                domainSelect.appendChild(option);
            });

            document.getElementById('dkimDomainSelect').value = g_selected_dkim_domain;

            console.log(result); // Optionally handle the result for debugging or additional actions
        } else {
            const errorResult = await response.json();
            alert('Error loadDkimCfgList: ' + errorResult.error);
        }
    } catch (error) {
        alert('An error occurred while loading the domain config list.');
    }
}



function onLoadDkimLCfg() {
    g_selected_dkim_domain = document.getElementById('dkimDomainSelect').value;
    if (!g_selected_dkim_domain) {
        document.getElementById('dkimCfgInfo').style.display = 'none';
        return;
    }
    loadDkimCfg(g_selected_dkim_domain);
}

async function loadDkimCfg(name) {
    try {
        const response = await fetch(`/load_dkim_cfg/${encodeURIComponent(name)}`);
        if (response.ok) {
            const result = await response.json();
            showDkimCfg(result);
            console.log(result); // Handle the result as needed
        } else {
            const errorResult = await response.json();
            alert('Error loadDkimCfg: ' + errorResult.error);
        }
    } catch (error) {
        alert(`An error occurred  ${error}.`);
    }
}

async function saveDkimCfg(data) {
    try {
        document.getElementById('dkim_status_log').innerText = "Please wait...";
        const response = await fetch('/save_dkim_cfg', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(data),
        });

        if (response.ok) {
            const result = await response.json();
            alert(result.message);
        } else {
            const errorResult = await response.json();
            //alert('Error saveDkimCfg: $(errorResult)');
            alert(`Error saveDkimCfg:: ${errorResult.message}`);
        }
        document.getElementById('dkim_status_log').innerText = "";
    } catch (error) {
        alert('An error occurred while saving the domain config.');
    }
}

async function removeDkimCfg(domain_name) {
    if (!domain_name) {
        alert('Campaign name is required.');
        return;
    }

    try {
        const response = await fetch('/remove_dkim_cfg', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ domain_name }),
        });

        if (response.ok) {
            const result = await response.json();
            loadDomainCfgList();
            document.getElementById('dkimCfgInfo').style.display = 'none';
            g_selected_dkim_domain = "";
            document.getElementById('dkimCfgInfo').style.display = 'none';
            alert(result.message);
        } else {
            const errorResult = await response.json();
            alert('Error: ' + errorResult.error);
        }
    } catch (error) {
        alert(`An error occurred while removing the domain config for ${domain_name}.`);
    }
}





function addNewDkimCfg() {

    document.getElementById('dkimCfgInfo').style.display = 'block';

    document.getElementById('dkim_domain_name').value = '';
    document.getElementById('cf_dkim_api_token').value = '';
    document.getElementById('dkim_imap_server_ip').value = "";
    document.getElementById('dkim_smtp_server_ip').value = "";


}


function showDkimCfg(dkimCfgData) {

    document.getElementById('dkimCfgInfo').style.display = 'block';

    document.getElementById('dkim_domain_name').value = dkimCfgData.dkim_domain_name;
    document.getElementById('cf_dkim_api_token').value = dkimCfgData.cf_dkim_api_token;
    document.getElementById('dkim_imap_server_ip').value = dkimCfgData.dkim_imap_server_ip;
    document.getElementById('dkim_smtp_server_ip').value = dkimCfgData.smtpIps.join('\n');



}


function isValidIPv4(address) {
    // Split the string by the '.' delimiter
    const octets = address.split('.');

    // Check if there are exactly 4 octets
    if (octets.length !== 4) {
        return false;
    }

    // Validate each octet
    for (let octet of octets) {
        // Check if the octet is a number and within the range 0-255
        if (!/^\d+$/.test(octet) || Number(octet) < 0 || Number(octet) > 255) {
            return false;
        }
    }

    return true; // All checks passed, it's a valid IPv4 address
}





function onSaveDkimCfg() {
    const dkim_domain_name = document.getElementById('dkim_domain_name').value;
    const cf_dkim_api_token = document.getElementById('cf_dkim_api_token').value;
    const dkim_imap_server_ip = document.getElementById('dkim_imap_server_ip').value;
    const dkim_smtp_server_ip = document.getElementById('dkim_smtp_server_ip').value;

    // Validate domain name
    if (!validatedomain_name(dkim_domain_name)) {
        alert("Invalid domain name");
        return;
    }

    // Check if Cloudflare API token is entered
    if (!cf_dkim_api_token) {
        alert("Please enter Cloudflare API token");
        return;
    }

    // Validate IMAP server IP
    if (!isValidIPv4(dkim_imap_server_ip)) {
        alert("Please enter a valid IMAP server IP");
        return;
    }

    // Validate and convert SMTP server IPs into an array
    let smtpIps = [];
    if (dkim_smtp_server_ip.trim() !== '') {
        const ipArray = dkim_smtp_server_ip.split('\n').map(ip => ip.trim()); // Split and trim each IP
        for (let ip of ipArray) {
            if (isValidIPv4(ip)) {
                smtpIps.push(ip);  // Add valid IPs to the array
            } else {
                alert(`Invalid IP address: ${ip}`);
                return;
            }
        }
    } else {
        alert("Please enter at least one valid SMTP server IP");
        return;
    }

    // Prepare data object with array of valid SMTP server IPs
    const data = {
        dkim_domain_name,
        cf_dkim_api_token,
        dkim_imap_server_ip,
        smtpIps  // Store the array of valid IPs
    };

    // Save DKIM configuration
    saveDkimCfg(data);
}



function onDelDkimCfg() {
    g_selected_dkim_domain = document.getElementById('dkimDomainSelect').value;
    if (!g_selected_dkim_domain) {
        alert("Please select a domain cfg firt");
        return;
    }

    removeDkimCfg(g_selected_dkim_domain);
}




