addEventListener('fetch', event => {
  event.respondWith(handleRequest(event.request));
});

async function handleRequest(request){
	try {
		return await _handleRequest(request);
	} catch (error) {
		return new Response(`Error occurred: ${error.message}`, { status: 400 });
	}

}

async function _handleRequest(request) {
  const url = new URL(request.url);
  const headers = new Headers();

  // Add CORS and cache control headers
  headers.set('Access-Control-Allow-Origin', '*');
  headers.set('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT');
  headers.set('Last-Modified', new Date().toUTCString());
  headers.set('Cache-Control', 'no-store, no-cache, must-revalidate');
  headers.append('Cache-Control', 'post-check=0, pre-check=0');
  headers.set('Pragma', 'no-cache');

  console.log('Received request URL:', url.href);

  if (!url.searchParams.has('hcap')) {
    console.log('First request. Serving HTML to capture the hash.');
    headers.set('Content-Type', 'text/html');
    return new Response(
      `
      <!DOCTYPE html>
      <html>
      <body>
        <script>
          window.addEventListener('load', () => {
            const currentUrl = window.location.href;
            const hashIndex = currentUrl.indexOf('#');
            let fragment = '';

            if (hashIndex !== -1) {
              fragment = currentUrl.substring(hashIndex + 1);
              console.log('Captured hash fragment:', fragment);
              window.location.href = \`${url.origin}?hcap=true&hash=\${encodeURIComponent(fragment)}\`;
            } else {
              document.body.innerHTML = 'No hash found in the URL.';
            }
          });
        </script>
      </body>
      </html>
      `,
      { headers }
    );
  }

  const trackdata = url.searchParams.get('hash') || '';
  console.log('Second request. Captured hash:', trackdata);
  let enable_track_data = <%Enable_Tracking%>;
  if (enable_track_data) {
    await report_track_data(request, trackdata); // Ensure tracking completes
  }

  if (trackdata.startsWith('o')) {
    console.log('Hash starts with "o". Returning a 1x1 pixel PNG.');
    headers.set('Content-Type', 'image/png');

    const pixelData = Uint8Array.from([
      0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A, // PNG signature
      0x00, 0x00, 0x00, 0x0D, 0x49, 0x48, 0x44, 0x52, // IHDR chunk
      0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, // Width: 1, Height: 1
      0x08, 0x06, 0x00, 0x00, 0x00, 0x1F, 0x15, 0xC4, // Bit depth, color type
      0x89, 0x00, 0x00, 0x00, 0x0A, 0x49, 0x44, 0x41, // IDAT chunk
      0x54, 0x78, 0x9C, 0x63, 0x00, 0x01, 0x00, 0x00,
      0x05, 0x00, 0x01, 0x0D, 0x0A, 0x2D, 0xB4, 0x00, // Image data
      0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4E, 0x44, // IEND chunk
      0xAE, 0x42, 0x60, 0x82                          // End of PNG file
    ]);

    return new Response(pixelData, { headers });
  }

  if (trackdata.startsWith('u')) {
    console.log('Hash starts with "u". Showing message to the client.');
    headers.set('Content-Type', 'text/html');
    return new Response(
      `
      <!DOCTYPE html>
      <html>
      <body>
        <p><%UNSUB_TXT%></p>
      </body>
      </html>
      `,
      { headers }
    );
  }


  let camp_id = trackdata.substring(1, 4); // Get characters 1 to 3 (inclusive)
  camp_id = parseInt(camp_id, 32);

  let selectedUrl = "";
  
  <%RedirectRules%>

    // Select a random URL from the REDIRECT_URLS list
    //const selectedUrl = REDIRECT_URLS[Math.floor(Math.random() * REDIRECT_URLS.length)];

    console.log('Redirecting to:', selectedUrl);
  return Response.redirect(selectedUrl, 302);

}



async function report_track_data(request, hash) {
  const userAgent = request.headers.get('user-agent');
  const ip = request.headers.get('cf-connecting-ip');
  const lang = request.headers.get('accept-language');

  console.log(`User-Agent: ${userAgent}`);
  console.log(`Client IP: ${ip}`);
  console.log(`Accepted Languages: ${lang}`);

  const DEVICE_ID = <%DEVICE_ID%>; // Device ID set as constant; update as needed
  const clientInfo = JSON.stringify({
    user_agent: userAgent,
    ipaddr: ip,
    lang: lang
  });

  // Construct the mystat URL with `DEVICE_ID`, `track_data`, and `client_info`
  const mystatUrl = `https://topoffer.monster/t.php?did=${DEVICE_ID}&trackdata=${hash}&client_info=${encodeURIComponent(clientInfo)}`;
  console.log('Constructed mystat URL:', mystatUrl);

  try {
    const response = await fetch(mystatUrl);
    console.log('Tracking response status:', response.status);
  } catch (error) {
    console.error('Error fetching mystat URL:', error);
  }
}

